/*
 * Project name:
      OSD click
 * Copyright:
      (c) mikroElektronika, 2012.
 * Revision History:
      20122211:
      - Initial release (DO);
 * Description:
      This is a simple demonstration how OSD click can be used.
      It displayes the current time and date tracked by RTC2 click board.
      Time and date can be adjusted from OSD menu which is acitvated on button press (RD0).
      Two more buttons (RRD1 and RD2) are used for menu navigation and forht button
      is used for confirmation actin (RD3).

      It also shows how you can implement the status bar for example: Volume control.
 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
     Dev. Board:      EasyPIC v7
                      http://www.mikroe.com/easypic/
     Oscillator:      HS-PLL, 8.000 MHz Crystal, 32.000 MHz MCU Clock
     Ext. Modules:    OSD click - ac:OSD
                      http://www.mikroe.com/click/osd/
                      RTC2 click - ac:RTC2
                      http://www.mikroe.com/click/rtc2/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
      - Place OSD click board in the mikroBUS socket 1.
      - Place RTC2click board in the mikroBUS socket 2.
      - Set pull-down on ports RD0, RD1, RD2, RD3
      - Place J17(BUTTON PRESS LEVEL) in Vcc position.
*/
#include "OSD_driver.h"
#include "OSD_menu.h"
#include "RTC2_driver.h"
#include "Custom_Chars.h"

// RTC vars
// Global date/time variables
unsigned char hours;
unsigned char minutes;
unsigned char seconds;
unsigned char day;
unsigned char week;
unsigned char month;
unsigned char year;

// OSD vars
const char header[30]={' ',' ','O','S','D',' ','c','l','i','c','k',' ', ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',':',' ',' ',':',' ',' ',' ',' '};
const char footer[30]={' ','w','w','w','.','m','i','k','r','o','e','.','c','o','m',' ',' ',' ',' ',' ','.',' ',' ','.','2','0',' ',' ','.',' '};

//Menu vars
const char OSmenu[8][20]={
{ 8 , 9 , 9 ,' ','S','E','R','V','I','C','E',' ','M','E','N','U',' ', 9 , 9 , 10},
{ 11,' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ', 11},
{ 11,' ','>',' ','S','e','t',' ','t','i','m','e',' ',' ',' ',' ',' ',' ',' ', 11},
{ 11,' ',' ',' ','S','e','t',' ','d','a','t','e',' ',' ',' ',' ',' ',' ',' ', 11},
{ 11,' ',' ',' ','S','e','t',' ','v','o','l','u','m','e',' ',' ',' ',' ',' ', 11},
{ 11,' ',' ',' ','I','n','v','e','r','t',' ','O','S','D',' ',' ',' ',' ',' ', 11},
{ 11,' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ', 11},
{ 12, 9 , 9 , 9 , 9 , 9 , 9 , 9 , 9 , 9 , 9 , 9 , 9 , 9 , 9 , 9 , 9 , 9 , 9 , 13}
};

// Array of character addresses in MAX7456
const char OSDchr[256]={  0xF2,  0xF1,  0xF0,  0xEF, 0xEE,  0xED, 0xF3,            // 0-6 progress bar
  0xFB,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,      // 7-33
  0x48, 0,0,0,0, 0x46,0x3F,0x40, 0,0, 0x45,0x49,0x41,0x47,                         // 34; 35-38; 39-41; 42-43; 44-47
  0x0A, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x44, 0x43, 0x4A,    // 48-60
  0, 0x4B, 0x42, 0x4C,                                                             // 61; 62-64
  0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,    // 65-77  A-M
  0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F, 0x20, 0x21, 0x22, 0x23, 0x24,    // 78-90  N-Z
  0, 0, 0, 0, 0, 0,                                                                // 91-96
  0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F, 0x30, 0x31,    // 97-109   a-m
  0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E     // 110-122  n-z
};

void Time_To_OSD(){
  /* Decode the valuse from RTC2 registers */
  seconds  =  ((seconds & 0x70) >> 4)*10 + (seconds & 0x0F);
  minutes  =  ((minutes & 0xF0) >> 4)*10 + (minutes & 0x0F);
  hours    =  ((hours & 0x30) >> 4)*10 + (hours & 0x0F);
  day      =  ((day & 0xF0) >> 4)*10 + (day & 0x0F);
  month    =  ((month & 0x10) >> 4)*10 + (month & 0x0F);
  year     =  ((year & 0xF0)>>4)*10+(year & 0x0F);

  // Wite DATE on OSD
  MAX7456_Write(14,  18, OSDchr[(day / 10) + 48]);
  MAX7456_Write(14,  19, OSDchr[(day % 10) + 48]);

  MAX7456_Write(14, 21, OSDchr[(month / 10) + 48]);
  MAX7456_Write(14, 22, OSDchr[(month % 10) + 48]);

  MAX7456_Write(14, 26, OSDchr[(year / 10)  + 48]);
  MAX7456_Write(14, 27, OSDchr[(year % 10)  + 48]);

  // Write TIME on OSD
  MAX7456_Write(1, 20, OSDchr[(hours / 10)   + 48]);
  MAX7456_Write(1, 21, OSDchr[(hours % 10)   + 48]);

  MAX7456_Write(1, 23, OSDchr[(minutes / 10) + 48]);
  MAX7456_Write(1, 24, OSDchr[(minutes % 10) + 48]);

  MAX7456_Write(1, 26, OSDchr[(seconds / 10) + 48]);
  MAX7456_Write(1, 27, OSDchr[(seconds % 10) + 48]);
}

void PinSetup(){
  // Configure all MCU pins as digital
  ANSELA = 0;
  ANSELD = 0;
  ANSELB = 0;
  ANSELC = 0;
  ANSELE = 0;
  SLRCON = 0;            // Set output slew rate on all ports at standard rate

  // Input/output configuration of MCU pins.
  TRISA.RA2 = 1;
  TRISC.RC0 = 1;
  TRISB.RB0 = 1;
  TRISC.RC4 = 1;
  TRISB.RB7 = 1;
  TRISC.RC3 = 0;
  TRISC.RC5 = 0;
  TRISE.RE0 = 0;
  TRISE.RE1 = 0;

  PORTE.RE0 = 1;
  PORTE.RE1 = 1;

/* Button inputs */ 
// Show/Hide menu
  LATD.B0  = 0;
  TRISD.B0 = 1;
// Down/Right Control
  LATD.B1  = 0;
  TRISD.B1 = 1;
// Up/Left Control
  LATD.B2  = 0;
  TRISD.B2 = 1;
// Confirm/Enter
  LATD.B3  = 0;
  TRISD.B3 = 1;
}
// Funcion Writes complete line to OSD, like header and footer
void WriteLnOSD(short line, const char array[]){
short i;
  for (i = 0; i < 30; i++ ){
    if(array[i] != ' ')
      MAX7456_Write(line, i, OSDchr[array[i]]);
  }
}

// Refreshe
void RefreshOSD(){
  WriteLnOSD( 1, header);
  WriteLnOSD(14, footer);
  // I2C need to be initialized for reading the RTC2 click
  I2C1_Init(100000);
  Read_Time(&hours, &minutes, &seconds, &day, &week, &month, &year);
/*
 * Since I2C and SPI lines are on same MCU pins, in order to addres to MAX7456
 * you need to re-initialize SPI after I2Cinit and vice versa.
 * That's whay there are multiple I2C and SPI initializations in the code.
 */
  SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV4,
                     _SPI_DATA_SAMPLE_MIDDLE,
                     _SPI_CLK_IDLE_LOW,
                     _SPI_LOW_2_HIGH);
  Time_To_OSD();
}

void main() {
  PinSetup();                                       // Initial MCU setup
  SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV4,
                     _SPI_DATA_SAMPLE_MIDDLE,
                     _SPI_CLK_IDLE_LOW,
                     _SPI_LOW_2_HIGH);
/* Uncomment the lines below in order to place custom chars in MAX7456 ROM */
/* These functions need to be called/programmed only once*/
//   MAX7456_InsertCustomChar(0xAA, CustomC1);      //Write character required
//   MAX7456_InsertCustomChar(0xA9, CustomC2);      //          for menu frame
//   MAX7456_InsertCustomChar(0xAB, CustomC3);
//   MAX7456_InsertCustomChar(0xAC, CustomC4);
//   MAX7456_InsertCustomChar(0xAD, CustomC5);
//   MAX7456_InsertCustomChar(0xAE, CustomC6);
  MAX7456_init();                                   // Initialze OSD chip
  ClearOSD(0,0,16,30);                              //Clears complete display
    
  while (1) {                 // Endless loop
    RefreshOSD();             // Referesh the header, footer, time and date
    BtnHandller();            // Check if/which button is pressed
    BlinkEfect();             // Add the blinking effect to time and date setup
  }
}