/**************************************************************************************************
* DS1307 Functions
**************************************************************************************************/
 // RTC Definitions
#define RTC_ADDR  0xD0
/**************************************************************************************************
* Read data from RTC DS1307
* input : addres of RTC register
* output: value of of RTC register
**************************************************************************************************/
unsigned char RTC_Read(unsigned char addr){
  unsigned char value;

  I2C1_Start();                   // Issue start signal
  I2C1_Wr(RTC_ADDR);              // Address DS1307, see DS1307 datasheet
  I2C1_Wr(addr);                  // Start from address 2
  I2C1_Start();                   // Issue repeated start signal
  I2C1_Wr(RTC_ADDR);              // Address DS1307 for reading R/W=1

  value = I2C1_Rd(0);             // Read seconds byte
  I2C1_Stop();                    // Issue stop signal

  return value;
}

/**************************************************************************************************
* Write data from DS1307
* input : addres of RTC register, value of of RTC register
**************************************************************************************************/
void RTC_Write(unsigned char addr, unsigned char value){
  I2C1_Start();         // Issue start signal
  I2C1_Wr(RTC_ADDR);    // Address DS1307
  I2C1_Wr(addr);        // Start from address
  I2C1_Wr(value);       // Write value to RTC register
  I2C1_Stop();          // Issue stop signal
}

/**************************************************************************************************
* Read time from RTC DS1307
* input : pointer to variables where RTC data will be stored
* output: variables with RTC data
**************************************************************************************************/
void Read_Time(unsigned char *p_hours, unsigned char *p_minutes, unsigned char *p_seconds,
               unsigned char *p_day, unsigned char *p_week, unsigned char *p_month, unsigned char *p_year){
  I2C1_Start();                // Issue start signal
  I2C1_Wr(RTC_ADDR);           // Address DS1307, see DS1307 datasheet
  I2C1_Wr(0);                  // Start from address 0
  I2C1_Repeated_Start();       // Issue repeated start signal
  I2C1_Wr(RTC_ADDR + 1);       // Address DS1307 for reading R/W=1

  *p_seconds = I2C1_Rd(1);     // Read seconds byte
  *p_minutes = I2C1_Rd(1);     // Read minutes byte
  *p_hours = I2C1_Rd(1);       // Read hours byte
  *p_week =I2C1_Rd(1);
  *p_day =I2C1_Rd(1);
  *p_month =I2C1_Rd(1);
  *p_year =I2C1_Rd(0);

  I2C1_Stop();                 // Issue stop signal
}

/**************************************************************************************************
* Write time to RTC DS1307
* input : variables with RTC data
**************************************************************************************************/
void Write_Time(unsigned char c_hours, unsigned char c_minutes, unsigned char c_seconds,
                unsigned char c_day, unsigned char c_week, unsigned char c_month, unsigned char c_year){
   I2C1_Start();                     // issue start signal
   I2C1_Wr(RTC_ADDR);                // address DS1307
   I2C1_Wr(0);                       // start from word at address (REG0)
   I2C1_Wr(0x80);                    // write $80 to REG0. (pause counter + 0 sec)

   I2C1_Wr(c_minutes);               // write 0 to minutes word to (REG1)
   I2C1_Wr(c_hours);                 // write 17 to hours word (24-hours mode)(REG2)
   I2C1_Wr(c_week);                  // write 2 - Monday (REG3)
   I2C1_Wr(c_day);                   // write 4 to date word (REG4)
   I2C1_Wr(c_month);                 // write 5 (May) to month word (REG5)
   I2C1_Wr(c_year);                  // write 01 to year word (REG6)
   I2C1_Stop();                      // issue stop signal

   I2C1_Start();                     // issue start signal
   I2C1_Wr(RTC_ADDR);                // address DS1307
   I2C1_Wr(0);                       // start from word at address 0
   I2C1_Wr(0 | c_seconds);           // write 0 to REG0 (enable counting + 0 sec)
   I2C1_Stop();                      // issue stop signal
}